# serial_over_ir.py - 22.01.2022
#
# UART Transmitter für den Pi Pico in asm_pio / MicroPython.
# Gibt das mit dem IR-Träger modulierte Signal aus.
# Die IR-Träger-Frequenz und die Baud-Rate sind beliebig einstelbar.
# Die Baud-Rate sollte <= IR-Freq // 10 betragen.
# Bis zu 9 Byte weren per TX-FIFO nonblocking versendet,
# alle weiteren Bytes blocken bis Platz in den Fifos frei wird.

from machine import Pin
from rp2 import PIO, StateMachine, asm_pio

# -----------------------------------------
@asm_pio(sideset_init=PIO.OUT_LOW)
# Hier wird der Träger für die IR-Led generiert
def ir_led():
    wrap_target()

    wait(0, irq, 4)  .side(0)  #  1 level low while waiting
    nop()            .side(1)  #  2 level high
    
    wrap()

# --------------------------------------------
@asm_pio(fifo_join=PIO.JOIN_TX,
         out_shiftdir=PIO.SHIFT_RIGHT,
         pull_thresh=8)

def uart_tx():
    
    wrap_target()
    pull()                      #  1 Warten auf Daten
                                #    jetzt sind Daten eingegangen ...
    irq(clear, 4)           [2] #  2 Das Startbit setzen 
    
    label("next_bit")           #    von hier an 2 Takte bis der Level
                                #    für das nächste Bit gesetzt wird
    
    out(x, 1)  [3]              #  3 das nächste Bit zum Prüfen nach X
    jmp(not_x, "notx")          #  4 X ist nicht gesetzt, springen
    
    irq(4)                      #  5 X ist gesetzt
    jmp("next")                 #  6
    
    label("notx")
    irq(clear, 4)           [1] #  7 X ist nicht gesetzt
    
    label("next")               #    hier kommen beide Zweige nach 2 Takten zusammen
    jmp(not_osre, "next_bit")   #  8 Sind noch weitere Bits vorhanden

    nop()                   [4] #  9 letztes Bit bis zum Ende halten
    irq(4)                  [6] # 10 Das Stopbit für 9 Takt setzen (1 + 6 + pull )
    
    wrap()


class Serial_Over_Ir(object):
    def __init__(self, ir_pin, baud=2400, ir_freq=38_000):
 
        self.sm0 = StateMachine(0, ir_led, freq=ir_freq*2, sideset_base=Pin(ir_pin))
        self.sm1 = StateMachine(1, uart_tx, freq=baud*8)

        self.sm0.restart()
        self.sm0.active(1)
        self.sm1.restart()
        self.sm1.active(1)
        
    def send(self, data):
        for d in data:
            self.sm1.put(d)
            
    def deactivate(self):
        self.sm0.active(0)
        self.sm1.active(0)
 